@@ -1,7 +1,7 @@
-from .face import Face, split_face, create_face_from_diagonals
+from .face import Face, split_face, create_face_from_diagonals, find_connected_face, find_face
 from .block import Block, rotate_block, reduce_blocks
 from .connectivity import get_outer_faces, find_matching_blocks, get_face_intersection, connectivity, connectivity_fast
-from .read import read_plot3D
+from .read import read_plot3D, read_ap_nasa
 from .write import write_plot3D
 from .differencing import find_edges, find_face_edges
 from .periodicity import periodicity, periodicity_fast, create_rotation_matrix, rotated_periodicity
