@@ -11,35 +11,6 @@ from copy import deepcopy
 from tqdm import trange
 import math
 
-def create_face(block:Block,imin:int,imax:int,jmin:int,jmax:int,kmin:int,kmax:int) -> Face:
-    """Creates a face/surface from IJK bounds. Face = either i is constant or j constant or k constant. 
-
-    Args:
-        block (Block): Block object containing X,Y,Z as 3 dimensional arrays 
-        imin (int): Minimum I-index
-        imax (int): Maximum I-index
-        jmin (int): Minimum J-index
-        jmax (int): Maximum J-index
-        kmin (int): Minimum K-index
-        kmax (int): Maximum K-index
-
-    Returns:
-        Face: Face object with vertices of I,J,K
-    """
-    f = Face(4)
-    if imin==imax:
-        for j in [jmin,jmax]:
-            for k in [kmin,kmax]:
-                f.add_vertex(block.X[imin,j,k], block.Y[imin,j,k], block.Z[imin,j,k],imin,j,k)
-    elif jmin==jmax:
-        for i in [imin,imax]:
-            for k in [kmin,kmax]:
-                f.add_vertex(block.X[i,jmin,k], block.Y[i,jmin,k], block.Z[i,jmin,k],i,jmin,k)
-    elif kmin==kmax:
-        for i in [imin,imax]:
-            for j in [jmin,jmax]:
-                f.add_vertex(block.X[i,j,kmin], block.Y[i,j,kmin], block.Z[i,j,kmin],i,j,kmin)
-    return f
 
 
 def periodicity_fast(blocks:List[Block],outer_faces:List[Face], matched_faces:List[Dict[str,int]], periodic_direction:str='k', rotation_axis:str='x',nblades:int=55):
@@ -200,14 +171,24 @@ def periodicity(blocks:List[Block],outer_faces:List[Dict[str,int]], matched_face
     periodic_faces = list()      # This is the output of the code 
     periodic_faces_export = list() 
     for o in outer_faces:
-        face = create_face(blocks[o['block_index']], o['IMIN'], o['IMAX'], o['JMIN'], o['JMAX'], o['KMIN'], o['KMAX'])
+        face = create_face_from_diagonals(blocks[o['block_index']], 
+            imin=o['IMIN'], imax=o['IMAX'], 
+            jmin=o['JMIN'], jmax=o['JMAX'], 
+            kmin=o['KMIN'], kmax=o['KMAX'])
         face.set_block_index(o['block_index'])
         outer_faces_all.append(face)
     for match_face_index,m in enumerate(matched_faces):
         if (match_face_index == 14):
             print("check")
-        face1 = create_face(blocks[m['block1']['block_index']], m['block1']['IMIN'], m['block1']['IMAX'], m['block1']['JMIN'], m['block1']['JMAX'], m['block1']['KMIN'], m['block1']['KMAX'])
-        face2 = create_face(blocks[m['block2']['block_index']], m['block2']['IMIN'], m['block2']['IMAX'], m['block2']['JMIN'], m['block2']['JMAX'], m['block2']['KMIN'], m['block2']['KMAX'])
+        face1 = create_face_from_diagonals(blocks[m['block1']['block_index']],
+            imin=int(m['block1']['IMIN']), imax=int(m['block1']['IMAX']), 
+            min=int(m['block1']['JMIN']), jmax=int(m['block1']['JMAX']),
+            kmin=int(m['block1']['KMIN']), kmax=int(m['block1']['KMAX']))
+        face2 = create_face_from_diagonals(blocks[m['block2']['block_index']], 
+            imin=int(m['block2']['IMIN']), imax=int(m['block2']['IMAX']), 
+            jmin=int(m['block2']['JMIN']), jmax=int(m['block2']['JMAX']), 
+            kmin=int(m['block2']['KMIN']), kmax=int(m['block2']['KMAX']))
+
         face1.set_block_index(m['block1']['block_index'])
         face2.set_block_index(m['block2']['block_index'])
         matched_faces_all.append(face1)
@@ -371,7 +352,7 @@ def periodicity(blocks:List[Block],outer_faces:List[Dict[str,int]], matched_face
     return periodic_faces_export, outer_faces_export, periodic_faces, outer_faces_all
 
 
-def rotated_periodicity(blocks:List[Block], matched_faces:List[Dict[str,int]], outer_faces:List[Dict[str,int]], rotation_angle:float, rotation_axis:str = "x"):
+def rotated_periodicity(blocks:List[Block], matched_faces:List[Dict[str,int]], outer_faces:List[Dict[str,int]], rotation_angle:float, rotation_axis:str = "x", ReduceMesh:bool=True):
     """Finds the peridocity/connectivity by rotating a block. This is a bit different from "periodicity" where you specify the periodic direction. 
         This method doesn't care about the direction as long as the angle you specify results in a match between the Left Face and the Right Face         
 
@@ -387,7 +368,7 @@ def rotated_periodicity(blocks:List[Block], matched_faces:List[Dict[str,int]], o
         outer_faces (List[Dict[str,int]]): List of outer faces in dictionary form
         rotation_angle (float): rotation angle in between blades in degrees. factor in an additional blade 
         rotation_axis (str, Optional): "x", "y", or "z" 
-
+        ReduceMesh (bool, Optional): True, reduces the mesh for faster matching
     
     periodic_faces, outer_faces_export, _, _ = rotated_periodicity(blocks,face_matches, outer_faces, rotation_angle=rotation_angle, rotation_axis = "x")
     
@@ -409,11 +390,12 @@ def rotated_periodicity(blocks:List[Block], matched_faces:List[Dict[str,int]], o
     """
     gcd_array = list()
     # Find the gcd of all the blocks 
-    for block_indx in range(len(blocks)):
-        block = blocks[block_indx]
-        gcd_array.append(math.gcd(block.IMAX-1, math.gcd(block.JMAX-1, block.KMAX-1)))
-    gcd_to_use = min(gcd_array) # You need to use the minimum gcd otherwise 1 block may not exactly match the next block. They all have to be scaled the same way.
-    blocks = reduce_blocks(deepcopy(blocks),gcd_to_use)
+    if ReduceMesh==True:
+        for block_indx in range(len(blocks)):
+            block = blocks[block_indx]
+            gcd_array.append(math.gcd(block.IMAX-1, math.gcd(block.JMAX-1, block.KMAX-1)))
+        gcd_to_use = min(gcd_array) # You need to use the minimum gcd otherwise 1 block may not exactly match the next block. They all have to be scaled the same way.
+        blocks = reduce_blocks(deepcopy(blocks),gcd_to_use)
 
     rotation_matrix = create_rotation_matrix(radians(rotation_angle),rotation_axis)
     
@@ -428,18 +410,18 @@ def rotated_periodicity(blocks:List[Block], matched_faces:List[Dict[str,int]], o
     periodic_faces = list()      # This is the output of the code 
     periodic_faces_export = list() 
     for o in outer_faces:
-        face = create_face(blocks[o['block_index']], int(o['IMIN']/gcd_to_use), int(o['IMAX']/gcd_to_use), 
-            int(o['JMIN']/gcd_to_use), int(o['JMAX']/gcd_to_use), int(o['KMIN']/gcd_to_use), int(o['KMAX']/gcd_to_use))
+        face = create_face_from_diagonals(blocks[o['block_index']], int(o['IMIN']/gcd_to_use), int(o['JMIN']/gcd_to_use), 
+            int(o['KMIN']/gcd_to_use), int(o['IMAX']/gcd_to_use), int(o['JMAX']/gcd_to_use), int(o['KMAX']/gcd_to_use))
         face.set_block_index(o['block_index'])
         outer_faces_all.append(face)
 
     for _,m in enumerate(matched_faces):
-        face1 = create_face(blocks_rotated[m['block1']['block_index']], int(m['block1']['IMIN']/gcd_to_use), int(m['block1']['IMAX']/gcd_to_use), 
-                            int(m['block1']['JMIN']/gcd_to_use), int(m['block1']['JMAX']/gcd_to_use), 
-                            int(m['block1']['KMIN']/gcd_to_use), int(m['block1']['KMAX']/gcd_to_use))
-        face2 = create_face(blocks[m['block2']['block_index']], int(m['block2']['IMIN']/gcd_to_use), int(m['block2']['IMAX']/gcd_to_use), 
-                            int(m['block2']['JMIN']/gcd_to_use), int(m['block2']['JMAX']/gcd_to_use), 
-                            int(m['block2']['KMIN']/gcd_to_use), int(m['block2']['KMAX']/gcd_to_use))
+        face1 = create_face_from_diagonals(blocks_rotated[m['block1']['block_index']], int(m['block1']['IMIN']/gcd_to_use), int(m['block1']['JMIN']/gcd_to_use), 
+                            int(m['block1']['KMIN']/gcd_to_use), int(m['block1']['IMAX']/gcd_to_use), 
+                            int(m['block1']['JMAX']/gcd_to_use), int(m['block1']['KMAX']/gcd_to_use))
+        face2 = create_face_from_diagonals(blocks[m['block2']['block_index']], int(m['block2']['IMIN']/gcd_to_use), int(m['block2']['JMIN']/gcd_to_use), 
+                            int(m['block2']['KMIN']/gcd_to_use), int(m['block2']['IMAX']/gcd_to_use), 
+                            int(m['block2']['JMAX']/gcd_to_use), int(m['block2']['KMAX']/gcd_to_use))
         face1.set_block_index(m['block1']['block_index'])
         face2.set_block_index(m['block2']['block_index'])
         matched_faces_all.append(face1)
@@ -650,10 +632,10 @@ def __periodicity_check__(face1:Face, face2:Face,block1:Block,block2:Block):
     df,split_face1,split_face2 = get_face_intersection(face1,face2,block1,block2)
 
     if len(df)>4:
-        f1 = create_face(block1,imin=df['i1'].min(),jmin=df['j1'].min(),kmin=df['k1'].min(), imax=df['i1'].max(),jmax=df['j1'].max(),kmax=df['k1'].max())
+        f1 = create_face_from_diagonals(block1,imin=df['i1'].min(),jmin=df['j1'].min(),kmin=df['k1'].min(), imax=df['i1'].max(),jmax=df['j1'].max(),kmax=df['k1'].max())
         f1.set_block_index(face1.blockIndex)
 
-        f2 = create_face(block2,imin=df['i2'].min(),jmin=df['j2'].min(),kmin=df['k2'].min(), imax=df['i2'].max(),jmax=df['j2'].max(),kmax=df['k2'].max())
+        f2 = create_face_from_diagonals(block2,imin=df['i2'].min(),jmin=df['j2'].min(),kmin=df['k2'].min(), imax=df['i2'].max(),jmax=df['j2'].max(),kmax=df['k2'].max())
         f2.set_block_index(face2.blockIndex)
         
         split_faces.extend(split_face1)
